@@ -22,6 +22,7 @@ import sys
 from typing import List
 
 import boto3
+import tldextract
 
 from .logger import get_logger
 from .client import get_secret
@@ -79,10 +80,12 @@ class LetsEncrypt:
             if not os.path.exists(acme_account):
                 os.makedirs(acme_account)
 
-            account_json = get_secret(f'{prefix}/otter/account.json', element=None, region=self.region)
+            account_json = get_secret(
+                f'{prefix}/otter/account.json', element=None, region=self.region)
             with open('account.json', 'w') as outfile:
                 outfile.write(account_json)
-            account_key = get_secret(f'{prefix}/otter/account.key', region=self.region)
+            account_key = get_secret(
+                f'{prefix}/otter/account.key', region=self.region)
             with open('account.key', 'w') as outfile:
                 outfile.write(account_key)
 
@@ -101,20 +104,53 @@ class LetsEncrypt:
             LOGGER.error(message)
             sys.exit(1)
 
-    def acme_development(self, csr: str) -> None:
+    def acme_development(self, csr: str) -> None:  # pragma: no cover
+        subdomain = tldextract.extract(
+            self.hostname).subdomain
         subprocess.call(
             '{directory}/acme.sh/acme.sh --upgrade -b dev'.format(directory=os.getenv('HOME')), shell=True)
-        subprocess.call('{directory}/acme.sh/acme.sh --set-default-ca --test --signcsr --csr {csr} --dns dns_aws --dnssleep 20 --challenge-alias {domain_validation} --preferred-chain "Fake LE Root X2" --force'.format(
-            directory=os.getenv('HOME'), csr=f'{csr}', domain_validation=self.subdelegate), shell=True)
+        subprocess.call('{directory}/acme.sh/acme.sh --set-default-ca --test --signcsr --csr {csr} --dns dns_aws --challenge-alias {domain_validation} --preferred-chain "Fake LE Root X2" --force'.format(
+            directory=os.getenv('HOME'), csr=f'{csr}', domain_validation=f'{subdomain}.{self.subdelegate}'), shell=True)
 
-    def acme_production(self, csr: str) -> None:
+    def acme_production(self, csr: str) -> None:  # pragma: no cover
+        subdomain = tldextract.extract(
+            self.hostname).subdomain
         subprocess.call(
             '{directory}/acme.sh/acme.sh --upgrade'.format(directory=os.getenv('HOME')), shell=True)
-        subprocess.call('{directory}/acme.sh/acme.sh --set-default-ca --server letsencrypt --signcsr --csr {csr} --dns dns_aws --dnssleep 20 --challenge-alias {domain_validation} --preferred-chain "ISRG Root X1" --force'.format(
-            directory=os.getenv('HOME'), csr=f'{csr}', domain_validation=self.subdelegate), shell=True)
+        subprocess.call('{directory}/acme.sh/acme.sh --set-default-ca --server letsencrypt --signcsr --csr {csr} --dns dns_aws --challenge-alias {domain_validation} --preferred-chain "ISRG Root X1" --force'.format(
+            directory=os.getenv('HOME'), csr=f'{csr}', domain_validation=f'{subdomain}.{self.subdelegate}'), shell=True)
 
-    def acme_local(self, csr: str) -> None:
+    def acme_local(self, csr: str) -> None:  # pragma: no cover
+        subdomain = tldextract.extract(
+            self.hostname).subdomain
         subprocess.call(
             '{directory}/.acme.sh/acme.sh --upgrade -b dev'.format(directory=os.getenv('HOME')), shell=True)
-        subprocess.call('{directory}/.acme.sh/acme.sh --set-default-ca --test --signcsr --csr {csr} --dns dns_aws --dnssleep 20 --challenge-alias {domain_validation} --preferred-chain "Fake LE Root X2" --force'.format(
-            directory=os.getenv('HOME'), csr=f'{csr}', domain_validation=self.subdelegate), shell=True)
+        subprocess.call('{directory}/.acme.sh/acme.sh --set-default-ca --test --signcsr --csr {csr} --dns dns_aws --challenge-alias {domain_validation} --preferred-chain "Fake LE Root X2" --force'.format(
+            directory=os.getenv('HOME'), csr=f'{csr}', domain_validation=f'{subdomain}.{self.subdelegate}'), shell=True)
+
+    def acme_private_ca_local(self, csr: str, acme_server_url: str) -> None:  # pragma: no cover
+        prefix = os.environ['PREFIX']
+        root_certificate = get_secret(
+            f'{prefix}/otter/root_certificate', element=None, region=self.region)
+        with open('root_ca.crt', 'w') as outfile:
+            outfile.write(root_certificate)
+
+        subdomain = tldextract.extract(
+            self.hostname).subdomain
+        subprocess.call(
+            '{directory}/.acme.sh/acme.sh --upgrade -b dev'.format(directory=os.getenv('HOME')), shell=True)
+        subprocess.call('{directory}/.acme.sh/acme.sh --set-default-ca --server {server} --ca-bundle root_ca.crt --signcsr --csr {csr} --dns dns_aws --challenge-alias {domain_validation} --force'.format(
+            directory=os.getenv('HOME'), server=acme_server_url, csr=f'{csr}', domain_validation=f'{subdomain}.{self.subdelegate}'), shell=True)
+
+    def acme_private_ca_production(self, csr: str, acme_server_url: str) -> None:  # pragma: no cover
+        prefix = os.environ['PREFIX']
+        root_certificate = get_secret(
+            f'{prefix}/otter/root_certificate', element=None, region=self.region)
+        with open('root_ca.crt', 'w') as outfile:
+            outfile.write(root_certificate)
+
+        subdomain = tldextract.extract(self.hostname).subdomain
+        subprocess.call(
+            '{directory}/acme.sh/acme.sh --upgrade'.format(directory=os.getenv('HOME')), shell=True)
+        subprocess.call('{directory}/acme.sh/acme.sh --set-default-ca --server {server} --ca-bundle root_ca.crt --signcsr --csr {csr} --dns dns_aws --challenge-alias {domain_validation} --force'.format(
+            directory=os.getenv('HOME'), server=acme_server_url, csr=f'{csr}', domain_validation=f'{subdomain}.{self.subdelegate}'), shell=True)
